#ifndef __scene_H__
#define __scene_H__
#include "rpLib.h"

struct _engine_parts {

	rpSceneNode *parent, *cylinder, *gear01, *gear02, *gear03, *gear04;
	float speed;
	bool going_up;
	
	// constructor
	_engine_parts() {
		going_up = true;
		speed = 0.01;
		parent = NULL;
		cylinder = NULL;
		gear01 = NULL;gear02 = NULL;gear03 = NULL;gear04 = NULL;

	}

	void trigger() {
		// rotate engine's gears
		if (gear01) gear01->rotateRel(E_AXIS_X, speed);
		if (gear02) gear02->rotateRel(E_AXIS_X, -speed);
		if (gear03) gear03->rotateRel(E_AXIS_X, speed);
		if (gear04) gear04->rotateRel(E_AXIS_X, speed);

		if (cylinder) {
			if (going_up)
				if (cylinder->position.y < 1.5)
					cylinder->moveRel(0.0, speed, 0.0);
				else going_up = false;

			if (!going_up)
				if (cylinder->position.y > 0.0)
					cylinder->moveRel(0.0, -speed, 0.0);
				else going_up = true;
		}


	}
};

void load_exported_scene(rpSceneManager *_smgr, bool hbuffer, bool hidetail)
{
	if (_smgr) {
		rpGeometryNode *_node = NULL;

		rpPhysicsCollision *_pspiral = NULL;
		// load geometry node
		if (!hidetail)
			_node = _smgr->newGeometryNode(NULL, "../media/spiral_M.oo", "../media/metal01.bmp",hbuffer);
		else
			_node = _smgr->newGeometryNode(NULL, "../media/spiral_L.oo", "../media/metal01.bmp",hbuffer);
		if (_node) {
			// user defined properties (pasted c++ code)
			_pspiral = _smgr->newPhysicsCollision(E_PCT_STATIC_GEOMETRY, _node->getCurrentGeometry());
			rpPhysicsNode *_pnode = _smgr->newPhysicsNode(_pspiral, _node);
		}
		
		// load geometry node
		if (!hidetail)
			_node = _smgr->newGeometryNode(NULL, "../media/spiral_M.oo", "../media/metal01.bmp");
		else
			_node = _smgr->newGeometryNode(NULL, "../media/spiral_L.oo", "../media/metal01.bmp");
		if (_node && _pspiral) {
			// user defined properties (pasted c++ code)
			rpPhysicsNode *_pnode = _smgr->newPhysicsNode(_pspiral, _node);
			rpMatrix mat;
			mat.move(0.0,-15.0,-5.0);
			mat.rotateY(-1.0);
			_pnode->setMatrix(&mat);
			
		}
		// a cup all objects fall into
		_node = _smgr->newGeometryNode(NULL, "../media/cup.oo", "../media/metal01.bmp", hbuffer);
		if (_node) {
			// user defined properties (pasted c++ code)
			rpPhysicsNode *_pnode = _smgr->newPhysicsNode(_smgr->newPhysicsCollision(E_PCT_STATIC_GEOMETRY, _node->getCurrentGeometry()), _node);
			rpMatrix mat;
			mat.move(6.0, -20.0, 5.0);
			_pnode->setMatrix(&mat);
			
			
		}
		// that horrible spawn thing
		/*_node = _smgr->newGeometryNode(NULL, "../media/spawn.oo", "../media/orange.bmp", hbuffer);
		if (_node) {
			_node->move(4.5, 15.0, 1.0);
			_node->rotate(E_AXIS_Y, 0.5);

		}*/

				// nice global light
		rpLightNode		*lN = _smgr->newLightNode(NULL);
		if (lN) {
			lN->move(0.0, 20.0,-10.0);
			lN->diffuseColor.set(1.0,1.0,1.0);
			lN->radius = 2000.0;
		}

	}
}
_engine_parts	load_engine(rpSceneManager *_smgr, bool hbuffer) {

	_engine_parts parts;
	rpSceneNode *_parent = _smgr->newSceneNode(NULL);
	parts.parent = _parent;
	
	
	// load geometry node
	rpGeometryNode *_node = _smgr->newGeometryNode(_parent, "../media/engine.oo", "../media/ALUM_3.bmp", hbuffer);
	if (_node) {
		_node->move(0.0, 0.0, -0.0905492);
		_node->rotate(0.0, 0.0, 0.0);
		// user defined properties (pasted c++ code)
		
	}
	// load geometry node
	parts.cylinder = _smgr->newGeometryNode(_parent, "../media/cylinder.oo", "../media/ALUM_3.bmp", hbuffer);
	if (parts.cylinder) {
		parts.cylinder->move(-0.000204057, 0.0036366, -0.0898075);
		
		// user defined properties (pasted c++ code)
		
	}
	// load geometry node
	parts.gear01 = _smgr->newGeometryNode(_parent, "../media/gear01.oo", "../media/BRNRUST1.bmp", hbuffer);
	if (parts.gear01) {
		parts.gear01->move(2.83191, 1.68255, 0.03699);
		parts.gear01->rotate(0.0,PI,0.0);
		// user defined properties (pasted c++ code)
		
	}
	
	return parts;

}

_engine_parts	load_engine_with_gear(rpSceneManager *_smgr, bool hbuffer) {

	_engine_parts parts;
	rpSceneNode *_parent = _smgr->newSceneNode(NULL);
	parts.parent = _parent;
	
	
	// load geometry node
	rpGeometryNode *_node = _smgr->newGeometryNode(_parent, "../media/engine.oo", "../media/ALUM_3.bmp", hbuffer);
	if (_node) {
		_node->move(0.0, 0.0, -0.0905492);
		_node->rotate(0.0, 0.0, 0.0);
		// user defined properties (pasted c++ code)
		
	}
	// load geometry node
	parts.cylinder = _smgr->newGeometryNode(_parent, "../media/cylinder.oo", "../media/ALUM_3.bmp", hbuffer);
	if (parts.cylinder) {
		parts.cylinder->move(-0.000204057, 0.0036366, -0.0898075);
		
		// user defined properties (pasted c++ code)
		
	}
	// load geometry node
	parts.gear01 = _smgr->newGeometryNode(_parent, "../media/gear01.oo", "../media/BRNRUST1.bmp", hbuffer);
	if (parts.gear01) {
		parts.gear01->move(2.83191, 1.68255, 0.03699);
		parts.gear01->rotate(0.0,PI,0.0);
		// user defined properties (pasted c++ code)
		
	}
	// load geometry node
	parts.gear02 = _smgr->newGeometryNode(_parent, "../media/gear02.oo", "../media/BRNRUST1.bmp", hbuffer);
	if (parts.gear02) {
		parts.gear02->move(-1.28734, 6.38811, 0.0485826);
		
		// user defined properties (pasted c++ code)
		
	}
	// load geometry node
	parts.gear03 = _smgr->newGeometryNode(_parent, "../media/gear00.oo", "../media/BRNRUST1.bmp", hbuffer);
	if (parts.gear03) {
		parts.gear03->move(3.76034, 4.05359, 0.0394333);
		
		// user defined properties (pasted c++ code)
		
	}
	// load geometry node
	parts.gear04 = _smgr->newGeometryNode(_parent, "../media/gear02.oo", "../media/BRNRUST1.bmp", hbuffer);
	if (parts.gear04) {
		parts.gear04->move(-1.29587, 6.38811, 0.0485826);
		parts.gear04->rotate(0.0,PI,0.0);
		// user defined properties (pasted c++ code)
		
	}

	return parts;

}


rpPhysicsNode *newBall(int which, rpSceneManager *smgr, bool hbuffer, bool hidetail) {

	switch (which) {
		case 0: {
			rpGeometryNode *_node = smgr->newGeometryNode(NULL, "../media/sphere.oo", "../media/tex.bmp", hbuffer);
			if (_node) {
				//_node->move(0.0,10.0,0.0);
				rpPhysicsNode *_pnode = smgr->newPhysicsNode(smgr->newPhysicsCollision(E_PCT_SPHERE, 0.5,0.5,0.5), _node);
				_pnode->weight = 100.0;
				rpMatrix mat;	mat.move(4.5,15.0,1.0);	_pnode->setMatrix(&mat);

				rpLightNode *_lnode = smgr->newLightNode(_node);
				_lnode->move(0.0,0.0,0.0);
				_lnode->diffuseColor.set(1.0,0.3,0.3);
				_lnode->radius = 50.0;

				return _pnode;
				
			}
			return NULL;
		}
		case 1: {
			rpGeometryNode *_node = smgr->newGeometryNode(NULL, "../media/sphere.oo", "../media/tex.bmp", true);
			if (_node) {
				//_node->move(0.0,10.0,0.0);
				rpPhysicsNode *_pnode = smgr->newPhysicsNode(smgr->newPhysicsCollision(E_PCT_CAPSULE,0.5, 1.0), _node);
				_pnode->weight = 180.0;
				//_pnode->setMassMatrix(0.2,1.0,1.0,1.0);
				rpMatrix mat;	mat.move(4.5,15.0,1.0);	_pnode->setMatrix(&mat);

				rpLightNode *_lnode = smgr->newLightNode(_node);
				_lnode->move(0.0,0.0,0.0);
				_lnode->diffuseColor.set(0.3,1.0,0.3);
				_lnode->radius = 15.0;

				return _pnode;

			}
			return NULL;
		}
		case 2: {
			rpGeometryNode *_node = smgr->newGeometryNode(NULL, "../media/sphere.oo", "../media/tex.bmp", true);
			if (_node) {
				//_node->move(0.0,10.0,0.0);
				rpPhysicsNode *_pnode = smgr->newPhysicsNode(smgr->newPhysicsCollision(E_PCT_SPHERE, 1.0,0.5,1.0), _node);
				_pnode->weight = 100.0;
				rpMatrix mat;	mat.move(4.5,15.0,1.0);	_pnode->setMatrix(&mat);

				rpLightNode *_lnode = smgr->newLightNode(_node);
				_lnode->move(0.0,0.0,0.0);
				_lnode->diffuseColor.set(0.3,0.5,1.0);
				_lnode->radius = 20.0;

				return _pnode;

			}
			return NULL;;
		}
	}
}
#endif
