
#include "rpLib.h"
#include "scene.h"
#include <string.h>
#include <sys\timeb.h>

rpSceneManager*		smgr = NULL;
rpVideoDriver*		driver = NULL; 
rpPhysicsNode *pN1 = NULL;
bool pause_physics = false;
bool freelook = false;
int m_x, m_y;

#define MAX_OBJECTS 3
#define MAX_ENGINES 3

rpCameraNode *camera = NULL;

class eventReceiver : public rpEventServer {


	virtual bool OnEvent(rpEvent event){

		if (event.EventType == E_ET_KEY_INPUT_EVENT) {
			switch (event.KeyInput.Key) {
				case KEY_KEY_W:
					{
						if (camera) {
							camera->transformation.move(0.0,0.0,1.0);
						}
					return true;
					}
				case KEY_KEY_S:
					{
						if (camera) {
							camera->transformation.move(0.0,0.0,-1.0);
						}
						return true;
					}
                case KEY_KEY_P:
					if (event.KeyInput.PressedDown)
					if (!pause_physics)	pause_physics = true;
					else pause_physics = false;
					break;
				case KEY_SPACE:
					if (event.KeyInput.PressedDown)
					if (!freelook)	freelook = true;
					else freelook = false;
					break;
				case KEY_KEY_E:
					
					if (smgr)smgr->updatePhysics();
					break;
				case KEY_KEY_R:
					if (event.KeyInput.PressedDown)
					if (pN1) {
						rpMatrix mat;
						mat.move(0.0,10.0,0.0);
						pN1->setMatrix(&mat);
					}
					break;
			}
		}
		else if (event.EventType == E_ET_MOUSE_INPUT_EVENT) {
			if (camera && freelook) {
				camera->transformation.rotateY((event.MouseInput.X - m_x) / 100.0);
				camera->transformation.rotateX((event.MouseInput.Y - m_y) / 100.0);
			}
			m_x = event.MouseInput.X;
			m_y = event.MouseInput.Y;
		
		
		}
	return false;
	}
	
};



int main(int argc, char *argv[])
{
	printf("Rocnikovy projekt, demonstracni aplikace\nPavel Gunia (xgunia01) 27.04.2005\n");
		
	int scr_w = 1024;
	int scr_h = 768;
	bool hbuffer = true;
	bool hidetail = false;
	    
    // go through arguments
	while (--argc) {
		if (strcmp(argv[argc], "-640x480") == 0) {
			scr_w = 640;
			scr_h = 480;
		}else
		if (strcmp(argv[argc], "-800x600") == 0) {
			scr_w = 800;
			scr_h = 600;
		}else
		if (strcmp(argv[argc], "-1024x768") == 0) {
			scr_w = 1024;
			scr_h = 768;
		}else
		if (strcmp(argv[argc], "-1280x1024") == 0) {
			scr_w = 1280;
			scr_h = 1024;
		}else
		if (strcmp(argv[argc], "-noHWB") == 0) {
			hbuffer = false;
		}
		else
		if (strcmp(argv[argc], "-hidetail") == 0) {
			hidetail = true;
		}
	}

	printf("%ix%i ", scr_w, scr_h);
	if (hbuffer)
		printf("(hardware accelerated vertex buffers) ");
	else
		printf("(pure software vertex buffers) ");

	if (hidetail)
		printf("(hi detail)\n");
	else printf("\n");
    
	printf("\n");

	// setup the application, input events too
	eventReceiver myReceiver;
	rpApplicationWin32	app(E_VD_OPENGL, scr_w, scr_h, false);
	app.setEventServer(&myReceiver);
	app.setCaption("Rocnikovy projekt, demonstracni aplikace ... starting");

	smgr = app.getSceneManager();
	driver = app.getVideoDriver();

	// create a physics world
	smgr->newPhysicsWorld();
	
	// default camera 
	camera = smgr->newCameraNode(NULL);
	camera->rotate(E_AXIS_X, 0.8);
	camera->move(0.0,0.0, -40.0);
    camera->updateTransformation();

	
	// load that nice scene
	load_exported_scene(smgr, hbuffer, hidetail);
	
	_engine_parts engine[MAX_ENGINES];	
	for (int i = 0; i < MAX_ENGINES; i++) {
		engine[i] = load_engine(smgr, hbuffer);
		engine[i].speed = 0.02f * (float)(i + 1);
		engine[i].parent->move(20.0,0.0,5.0 - (i * 5.0));
	}

	_engine_parts cup_engine = load_engine_with_gear(smgr, hbuffer);
	cup_engine.speed = 0.03f;
	cup_engine.parent->move(11.0,-20.0,12.0);
	cup_engine.parent->rotate(0.0,-PI / 2.0,0.0);

	
	// create a physics nodes
	int last_node = 0;
    rpPhysicsNode *ball[MAX_OBJECTS];
	
	ball[0] = newBall(0, smgr, hbuffer,hidetail);
	ball[1] = NULL;
	ball[2] = NULL;

	// create the top cup
	rpGeometryNode *top_cup = smgr->newGeometryNode(NULL, "../media/cup.oo", "../media/orange.bmp");
	top_cup->move(12.0, -16.0, 5.0);
	top_cup->rotate(E_AXIS_Z, 3.147);
	
	bool all_down = false;
	float top_cup_x = 12.0;

	// statistics
	int old_fcount = 0;
	c8 w_caption[256];
	// statistics per second
	driver->setStatsInterval(1000);


	while (app.run())
	{
		// the whole loop
		smgr->beginScene(true, true, rpColorf(0.3f,0.4f,0.5f));

		if (!pause_physics)
			smgr->updatePhysics();

		// balls are falling down the spiral
		if (!all_down) {
			if (ball[0])
				if ((ball[0]->getSceneNode()->absolute_transformation.getPosY() < 0.0) && !ball[1]) {
					ball[1] = newBall(1, smgr, hbuffer, hidetail);
				}	
			if (ball[1])
				if ((ball[1]->getSceneNode()->absolute_transformation.getPosY() < 0.0) && !ball[2]) {
					ball[2] = newBall(2, smgr, hbuffer, hidetail);
				}
			
			if (ball[2])
				if (ball[2]->getSceneNode()->absolute_transformation.getPosY() < -15.0)
					all_down = true;

			// slide the top cup back if needed
			if (top_cup_x < 12.0) {
				top_cup->move(top_cup_x, -15.0, 5.0);	
				top_cup_x += 0.01;
				cup_engine.speed = 0.01;
				cup_engine.trigger();
			}

		} else	// all balls are in the cup, spawn them up again
		{
			if (top_cup_x > 6.0) {
				top_cup->move(top_cup_x, -15.0, 5.0);	
				top_cup_x -= 0.05;
				cup_engine.speed = 0.05;
				cup_engine.trigger();
			} else {	// cup sliding finished

				// restart the whole animation
				delete ball[0];
				delete ball[1];
				delete ball[2];
				ball[0] = newBall(0, smgr, hbuffer, hidetail);
				ball[1] = NULL;
				ball[2] = NULL;
				all_down = false;

			}
			
		}

		// rotate enginew
		for (int i = 0; i < MAX_ENGINES; i++)
			engine[i].trigger();
		

		
		smgr->drawAll();
		smgr->endScene();

			
		// do some performance checking
		if (driver->getLastFramesCount() != old_fcount) {
			sprintf(w_caption, "Rocnikovy projekt, demonstracni aplikace     FPS: %f", driver->getLastFramesPerSec());
			old_fcount = driver->getLastFramesCount();
			app.setCaption(w_caption);
		}
		
		

		
		
	}

	return 0;
}
