
/*****************************************************************************\
 *
 * rpLib	- graphic library for rapid development of graphical applications
 *
 * Author:	Pavel Gunia (xgunia01 @ stud.fit.vutbr.cz)
 *
 * ----------------------------------------------------------------------------
 *
 * THIS SOFTWARE IS NOT COPYRIGHTED
 *
 * This source code is offered for use in the public domain.
 * You may use, modify or distribute it freely.
 *
 * This source code is distributed in the hope that it will be useful but
 * WITHOUT ANY WARRANTY.  ALL WARRANTIES, EXPRESS OR IMPLIED ARE HEREBY
 * DISCLAIMED.  This includes but is not limited to warranties of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *
 * If you find the source code useful, authors will kindly welcome
 * if you give them credit and keep their names with their source code,
 * but do not feel to be forced to do so.
 *
\*****************************************************************************/

#ifndef RPAPPLICATION_H
#define RPAPPLICATION_H

#include "rpBase.h"
#include "rpVideoDriver.h"
#include "rpSceneManager.h"
#include "rpEventServer.h"

// the engine itself
class rpApplication :
	public rpBase
{
public:
		
	rpApplication(void):video_driver(0), scene_manager(0),event_server(NULL){}
	virtual ~rpApplication(void){}	// TODO:drop all

protected:
	// all we need to execute the rendering procedure
	rpVideoDriver	*video_driver;
	rpSceneManager	*scene_manager;

public:
	
	// virtual methods
	
	
	// set window caption
	virtual void setCaption(const c8*) = 0;
	
	// run the application loop
	virtual bool run() = 0;
	




	
	// 'get' methods
	rpVideoDriver	*getVideoDriver() {return video_driver;}
	rpSceneManager	*getSceneManager() {return scene_manager;}

	// evennts handling
	rpEventServer	*event_server;
	void	setEventServer(rpEventServer *eS) {
		event_server = eS; }
	void	postEvent(rpEvent &e) {
		if (event_server)
			event_server->OnEvent(e);
	}

public:
	// 'close' methods
	void close() {
		if (video_driver) video_driver->drop();
		if (scene_manager) scene_manager->drop();
	}


};


#endif