/*****************************************************************************\
 *
 * rpLib	- graphic library for rapid development of graphical applications
 *
 * Author:	Pavel Gunia (xgunia01 @ stud.fit.vutbr.cz)
 *
 * ----------------------------------------------------------------------------
 *
 * THIS SOFTWARE IS NOT COPYRIGHTED
 *
 * This source code is offered for use in the public domain.
 * You may use, modify or distribute it freely.
 *
 * This source code is distributed in the hope that it will be useful but
 * WITHOUT ANY WARRANTY.  ALL WARRANTIES, EXPRESS OR IMPLIED ARE HEREBY
 * DISCLAIMED.  This includes but is not limited to warranties of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *
 * If you find the source code useful, authors will kindly welcome
 * if you give them credit and keep their names with their source code,
 * but do not feel to be forced to do so.
 *
\*****************************************************************************/


#ifndef RPBASE_H
#define RPBASE_H

#include "rpTypes.h"

class rpBase
	{
	public:

		// Constructor.
		rpBase()
			: ReferenceCounter(1), DebugName(0)
		{
		}

		// Destructor.
		virtual ~rpBase()
		{
		}

		// Grabs the object. Increments the reference counter by one.
		void grab() { ++ReferenceCounter; }

		// Drops the object. Decrements the reference counter by one.
		bool drop()
		{
			--ReferenceCounter;
			if (!ReferenceCounter)
			{
				delete this;
				return true;
			}

			return false;
		}

		// Returns the debug name of the object.
		const c8* getDebugName() const
		{
			return DebugName;
		} 

	protected:

		// Sets the debug name of the object. 
		void setDebugName(const char* newName)
		{
			DebugName = newName;
		}

	private:

		i32	ReferenceCounter;
		const c8* DebugName;
	};


#endif
