/*****************************************************************************\
 *
 * rpLib	- graphic library for rapid development of graphical applications
 *
 * Author:	Pavel Gunia (xgunia01 @ stud.fit.vutbr.cz)
 *
 * ----------------------------------------------------------------------------
 *
 * THIS SOFTWARE IS NOT COPYRIGHTED
 *
 * This source code is offered for use in the public domain.
 * You may use, modify or distribute it freely.
 *
 * This source code is distributed in the hope that it will be useful but
 * WITHOUT ANY WARRANTY.  ALL WARRANTIES, EXPRESS OR IMPLIED ARE HEREBY
 * DISCLAIMED.  This includes but is not limited to warranties of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *
 * If you find the source code useful, authors will kindly welcome
 * if you give them credit and keep their names with their source code,
 * but do not feel to be forced to do so.
 *
\*****************************************************************************/



#ifndef RPEVENTSERVER_H
#define RPEVENTSERVER_H

#include "rpTypes.h"

//! Enumeration for all event types there are.
enum E_EVENT_TYPE
{
	//! A mouse input event.
	E_ET_MOUSE_INPUT_EVENT = 0,

	//! A key input evant.
	E_ET_KEY_INPUT_EVENT,
};

//! Enumeration for all mouse input events
enum E_MOUSE_INPUT_EVENT
{
	//! Left mouse button was pressed down.
	E_MIE_LMOUSE_PRESSED_DOWN = 0,

	//! Right mouse button was pressed down.
	E_MIE_RMOUSE_PRESSED_DOWN,

	//! Middle mouse button was pressed down.
	E_MIE_MMOUSE_PRESSED_DOWN,

	//! Left mouse button was left up.
	E_MIE_LMOUSE_LEFT_UP,

	//! Right mouse button was left up.
	E_MIE_RMOUSE_LEFT_UP,

	//! Middle mouse button was left up.
	E_MIE_MMOUSE_LEFT_UP,

	//! The mouse cursor changed its position.
	E_MIE_MOUSE_MOVED,

	//! The mouse wheel was moved. Use Wheel value in event data to find out 
	//! in what direction and how fast.
	E_MIE_MOUSE_WHEEL, 
};



//! Struct for holding event data. An event can be a gui, mouse or keyboard event.
struct rpEvent
{
	E_EVENT_TYPE EventType;

	union
	{
		struct
		{
			i32 X, Y; // mouse positions;

			//! mouse wheel delta, usually 1.0 or -1.0. Only valid if event was
			//! E_MIE_MOUSE_WHEEL
			f32 Wheel; 

			//! type of more event
			E_MOUSE_INPUT_EVENT Event;
		} MouseInput;

		struct
		{
			wchar_t Char; // Character corresponding to the key (0, if not a character)
			E_KEY_CODE Key; // Key pressed
			bool PressedDown; // if not pressed, then left up
			bool Shift; // true if shift was also pressed
			bool Control; // true if ctrl was also pressed
		} KeyInput;
	};

};

//! Interface of an object wich can receive events.
class rpEventServer
{
public:

	virtual ~rpEventServer() {};

	//! called if an event happened. returns true if event was processed
	virtual bool OnEvent(rpEvent event) = 0;
};


#endif

