/*****************************************************************************\
 *
 * rpLib	- graphic library for rapid development of graphical applications
 *
 * Author:	Pavel Gunia (xgunia01 @ stud.fit.vutbr.cz)
 *
 * ----------------------------------------------------------------------------
 *
 * THIS SOFTWARE IS NOT COPYRIGHTED
 *
 * This source code is offered for use in the public domain.
 * You may use, modify or distribute it freely.
 *
 * This source code is distributed in the hope that it will be useful but
 * WITHOUT ANY WARRANTY.  ALL WARRANTIES, EXPRESS OR IMPLIED ARE HEREBY
 * DISCLAIMED.  This includes but is not limited to warranties of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *
 * If you find the source code useful, authors will kindly welcome
 * if you give them credit and keep their names with their source code,
 * but do not feel to be forced to do so.
 *
\*****************************************************************************/


#ifndef RPGEOMETRYBUFFER_H
#define RPGEOMETRYBUFFER_H

#include "rpBase.h"
#include <stdio.h>
#include <string.h>

struct sStandard3f
{
	//! default constructor
	sStandard3f() {
		x = 0.0f; y = 0.0f; z = 0.0f;
		nx = 0.0f; ny = 0.0f; nz = 0.0f;
		ca = 0.0f; cr = 0.0f; cg = 0.0f; cb = 0.0f;
		u = 0.0f; v = 0.0f;
	}

	//! constructor
	sStandard3f(f32 _x, f32 _y, f32 _z, f32 _nx, f32 _ny, f32 _nz, f32 _cr, f32 _cg, f32 _cb, f32 _u, f32 _v) {
		x = _x; y = _y; z = _z;
		nx = _nx; ny = _ny; nz = _nz;
        ca = 1.0; cr = _cr; cg = _cg; cb = _cb;
		if (cr > 1.0) cr = 1.0;
		if (cg > 1.0) cg = 1.0;
		if (cb > 1.0) cb = 1.0;
		u = _u; v = _v;
	
	}

	f32 x,y,z;			// position
	f32 nx,ny,nz;			// normals
	f32 ca,cr,cg,cb;		// colors
	f32 u,v;				// texcoords
};
struct sFace3f {
	sFace3f() {index[0] = 0; index[1] = 0; index[2] = 0; }
	
	i32 index[3];
};

enum E_GEOBUFFER_TYPE {
	E_GB_PLAIN = 0,
	E_GB_INDEXED};

enum E_ALLOC_TYPE {
	E_AT_VERTICES = 0,
	E_AT_NORMALS,
	E_AT_COLORS,
	E_AT_TEXCOORDS};

enum E_GDRIVER_TYPE {
	E_GEOBUFFER_OPENGL = 0,
	E_GEOBUFFER_DIRECTX	};

class rpGeometryBuffer :
	public rpBase
{
public:

	// constructor to create a plain mesh
	rpGeometryBuffer(i32 DT){
		driver_type = DT;

		// disable hardware by default
		hardware_buffer = false;
	}
	virtual ~rpGeometryBuffer(void){}


	// creation methods
	virtual void addVertex(sStandard3f &) = 0;
	virtual void addFace(i32, i32, i32) = 0;
	virtual void addIndex(i32) = 0;

	// 'get' methods
	virtual sStandard3f		getVertex(ui32) = 0;
	virtual sFace3f			getFace(ui32) = 0;

	virtual i32				getVertexCount() = 0;
	virtual i32				getIndexCount() = 0;


	
	
	// allocation
	virtual void reallocate(E_ALLOC_TYPE, ui32) = 0;



	// load from file
	bool getFromFile(const c8 *_file)
	{
		FILE *in_file = fopen(_file, "r");
		if (in_file) {
			c8 str_token[256];
			
			fscanf(in_file, "%s", &str_token);
			if (strcmp(str_token, "G_PLAIN") == 0) {

				// load the plain geometry buffer
				i32 size = 0;
				fscanf(in_file, "%i", &size);
							
				// reallocate buffers so it doesn't happen during loading
				if (size > 0) {
					reallocate(E_AT_VERTICES, size * 3);
					reallocate(E_AT_NORMALS, size * 3);
					reallocate(E_AT_TEXCOORDS, size * 2);
				}

				begin();
				
				// load the file
				for (i32 i = 0; i < size * 3; i++) {
					f32 e_v[3];
					f32 e_n[3];
					f32 e_t[2];

					
					fscanf(in_file, "%s", &str_token);
					// vertices
					for (i32 e = 0; e < 3; e++) {
						// unexpected end of file
						if (feof(in_file)) return false;
						// else
						fscanf(in_file, "%f", &e_v[e]);
					}
										
					fscanf(in_file, "%s", &str_token);
					// normals
					for (i32 e = 0; e < 3; e++) {
						// unexpected end of file
						if (feof(in_file)) return false;
						// else
						fscanf(in_file, "%f", &e_n[e]);
					}
					
					fscanf(in_file, "%s", &str_token); 
					// texcoords
					for (i32 e = 0; e < 2; e++) {
						// unexpected end of file
						if (feof(in_file)) return false;
						// else
						fscanf(in_file, "%f", &e_t[e]);
					}
					
					addVertex(sStandard3f(e_v[0], e_v[1], e_v[2], e_n[0], e_n[1], e_n[2], 1.0f,1.0f,1.0f, e_t[0], e_t[1]));
					
				}

				// finished loading
				type = E_GB_PLAIN;
				finish();
				

			}
			else 
			if (strcmp(str_token, "G_INDEXED") == 0) {


			}
			fclose(in_file);
		}
		return true;
	}

	// prepare for inserting geometry data
	virtual void begin() = 0;
	// building finished
	virtual void finish() = 0;

	E_GEOBUFFER_TYPE type;
	i32 driver_type;
	
	// whether load into device's memory
	bool			hardware_buffer;

};

#endif

