/*****************************************************************************\
 *
 * rpLib	- graphic library for rapid development of graphical applications
 *
 * Author:	Pavel Gunia (xgunia01 @ stud.fit.vutbr.cz)
 *
 * ----------------------------------------------------------------------------
 *
 * THIS SOFTWARE IS NOT COPYRIGHTED
 *
 * This source code is offered for use in the public domain.
 * You may use, modify or distribute it freely.
 *
 * This source code is distributed in the hope that it will be useful but
 * WITHOUT ANY WARRANTY.  ALL WARRANTIES, EXPRESS OR IMPLIED ARE HEREBY
 * DISCLAIMED.  This includes but is not limited to warranties of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *
 * If you find the source code useful, authors will kindly welcome
 * if you give them credit and keep their names with their source code,
 * but do not feel to be forced to do so.
 *
\*****************************************************************************/


#ifndef RPGLGEOMETRYBUFFER_H
#define RPGLGEOMETRYBUFFER_H


#include "rpGeometryBuffer.h"
#include "rpArray.h"

#include <stdio.h>



struct	sVertex3f {
	f32	x,y,z;
	sVertex3f() {x = 0.0;y = 0.0; z = 0.0;}
	sVertex3f(f32 _x, f32 _y, f32 _z) {x = _x;y = _y; z = _z;}
};
struct	sVertex2f {
	f32	u,v;
	sVertex2f() {u = 0.0;v = 0.0;}
	sVertex2f(f32 _u, f32 _v) {u = _u;v = _v;}
};

class rpVideoDriverOpenGL;

class rpGlGeometryBuffer :
	public rpGeometryBuffer
{
public:
	
	rpGlGeometryBuffer(i32 initial_vertices, i32 initial_indices, i32 T, rpVideoDriverOpenGL *D);
	
	virtual ~rpGlGeometryBuffer(void);

	// creation methods
	virtual void addVertex(sStandard3f &);
	virtual void addFace(i32, i32, i32);
	virtual void addIndex(i32);


	// 'get' methods
	virtual sStandard3f		getVertex(ui32);
	virtual sFace3f			getFace(ui32);

	virtual i32				getVertexCount();
	virtual i32				getIndexCount();

	// allocation
	virtual void reallocate(E_ALLOC_TYPE, ui32);

	// prepare for inserting geometry data
	virtual void begin();
	// building finished
	virtual void finish();



	rpArray<sVertex3f>	vertices;
	rpArray<sVertex3f>	normals;
	rpArray<sVertex3f>	colors;
	rpArray<sVertex2f>	texcoords;
	rpArray<unsigned short>	indices;

	E_GEOBUFFER_TYPE type;

	rpVideoDriverOpenGL	*device;

	// in case of a HW buffer
	i32				buffer_ID;
	
};



#endif