/*****************************************************************************\
 *
 * rpLib	- graphic library for rapid development of graphical applications
 *
 * Author:	Pavel Gunia (xgunia01 @ stud.fit.vutbr.cz)
 *
 * ----------------------------------------------------------------------------
 *
 * THIS SOFTWARE IS NOT COPYRIGHTED
 *
 * This source code is offered for use in the public domain.
 * You may use, modify or distribute it freely.
 *
 * This source code is distributed in the hope that it will be useful but
 * WITHOUT ANY WARRANTY.  ALL WARRANTIES, EXPRESS OR IMPLIED ARE HEREBY
 * DISCLAIMED.  This includes but is not limited to warranties of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *
 * If you find the source code useful, authors will kindly welcome
 * if you give them credit and keep their names with their source code,
 * but do not feel to be forced to do so.
 *
\*****************************************************************************/


#ifndef RPMATH_H
#define RPMATH_H

#include <math.h>



//-------------------------------------------------------------------
#ifndef PI
#define PI (3.14159265f)
#endif


#ifndef TINY
#define TINY (0.0000001)
#endif

#define rp_max(a,b)      (((a) > (b)) ? (a) : (b))
#define rp_min(a,b)      (((a) < (b)) ? (a) : (b))
#define rp_abs(a)        (((a)<0.0f) ? (-(a)) : (a))
#define rp_sgn(a)        (((a)<0.0f) ? (-1) : (1))


#define rp_deg2rad(d)    (((d)*PI)/180.0f)
#define rp_rad2deg(r)    (((r)*180.0f)/PI)
#define rp_sin(x) (float(sin(x)))
#define rp_cos(x) (float(cos(x)))

static inline float rp_acos(float x)
{
    if(x >  1.0f) x =  1.0f;
    if(x < -1.0f) x = -1.0f;
    return (float)acos(x);
}

static inline float rp_asin(float x)
{
    if(x >  1.0f) x =  1.0f;
    if(x < -1.0f) x = -1.0f;
    return (float)asin(x);
}

static inline float rp_sqrt(float x)
{
    if (x<0.0f) x=(float)0.0f;
    return (float) sqrt(x);
}

//-------------------------------------------------------------------
//  n_fequal()
//  Return true if (f0==f1)
//-------------------------------------------------------------------
static inline bool rp_fequal(float f0, float f1, float tol) {
    float f = f0 - f1;
    if ((f>(-tol)) && (f<tol)) return true;
    else                       return false;
}

//-------------------------------------------------------------------
//  n_fless()
//  Return true if (f0<f1).
//-------------------------------------------------------------------
static inline bool rp_fless(float f0, float f1, float tol) {
    if ((f0 - f1) < tol)
		return true;
    else
		return false;
}

//-------------------------------------------------------------------
//  n_fgreater()
//  Return true if (f0>f1)
//-------------------------------------------------------------------
static inline bool rp_fgreater(float f0, float f1, float tol) {
    if ((f0 - f1) > tol)return true;
    else				return false;
}

//-------------------------------------------------------------------
//  fast float to int conversion (always truncates)
//  see http://www.stereopsis.com/FPU.html for a discussion.
//  NOTE: this works only on x86 endian machines.
//-------------------------------------------------------------------
static inline long rp_ftol(float val)
{
    double v = double(val) + (68719476736.0*1.5);
    return ((long*)&v)[0] >> 16;
}

//-------------------------------------------------------------------
#endif










