/*****************************************************************************\
 *
 * rpLib	- graphic library for rapid development of graphical applications
 *
 * Author:	Pavel Gunia (xgunia01 @ stud.fit.vutbr.cz)
 *
 * ----------------------------------------------------------------------------
 *
 * THIS SOFTWARE IS NOT COPYRIGHTED
 *
 * This source code is offered for use in the public domain.
 * You may use, modify or distribute it freely.
 *
 * This source code is distributed in the hope that it will be useful but
 * WITHOUT ANY WARRANTY.  ALL WARRANTIES, EXPRESS OR IMPLIED ARE HEREBY
 * DISCLAIMED.  This includes but is not limited to warranties of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *
 * If you find the source code useful, authors will kindly welcome
 * if you give them credit and keep their names with their source code,
 * but do not feel to be forced to do so.
 *
\*****************************************************************************/


#ifndef RPMATRIX_H
#define RPMATRIX_H

#include <math.h>
#include <string.h>


static float matrix_ident[16] = {
    1.0f, 0.0f, 0.0f, 0.0f,
    0.0f, 1.0f, 0.0f, 0.0f,
    0.0f, 0.0f, 1.0f, 0.0f,
    0.0f, 0.0f, 0.0f, 1.0f,
};


//! collumn-major order matrix (OpenGl native)
//! simple but useful
class rpMatrix {
public:
    float m11, m12, m13, m14;
    float m21, m22, m23, m24;
    float m31, m32, m33, m34;
    float m41, m42, m43, m44;
  
    inline rpMatrix();
    inline rpMatrix(float, float, float, float,
                     float, float, float, float,
                     float, float, float, float,
                     float, float, float, float);
	inline rpMatrix(rpMatrix &M)
	{	memcpy(this, &M, sizeof(matrix_ident)); }

    inline void copyTo(rpMatrix &);
    inline void set(float, float, float, float,
                    float, float, float, float,
                    float, float, float, float,
                    float, float, float, float);
    inline void  ident(void);
    inline float det(void);
    inline void  invert(void);

    inline void mult(const rpMatrix &);
    inline void multSimple(const rpMatrix &);
	inline void transpose(rpMatrix &);


    inline void translate(float, float, float);
	inline void move(float, float, float);
    inline void scale(float, float, float);
    inline void rotateX(float);
    inline void rotateY(float);
    inline void rotateZ(float);

	inline float getPosX() {
		return m41;
	}
	inline float getPosY() {
		return m42;
	}
	inline float getPosZ() {
		return m43;
	}

    inline void setRvec(float, float, float, float);
	
	float *data() {return (float *)this;}
 
};

//-------------------------------------------------------------------
#define m0(y,x) m_0[(y<<2)+x]
#define m1(y,x) m_1[(y<<2)+x]

//-------------------------------------------------------------------
inline rpMatrix::rpMatrix()
{
     memcpy(this,matrix_ident,sizeof(matrix_ident));
}
//-------------------------------------------------------------------
inline rpMatrix::rpMatrix(float _m11, float _m12, float _m13, float _m14,
                            float _m21, float _m22, float _m23, float _m24,
                            float _m31, float _m32, float _m33, float _m34,
                            float _m41, float _m42, float _m43, float _m44)
{
    m11=_m11; m12=_m12; m13=_m13; m14=_m14;
    m21=_m21; m22=_m22; m23=_m23; m24=_m24;
    m31=_m31; m32=_m32; m33=_m33; m34=_m34;
    m41=_m41; m42=_m42; m43=_m43; m44=_m44;
}
//-------------------------------------------------------------------
inline void rpMatrix::set(float _m11, float _m12, float _m13, float _m14,
                           float _m21, float _m22, float _m23, float _m24,
                           float _m31, float _m32, float _m33, float _m34,
                           float _m41, float _m42, float _m43, float _m44)
{
    m11=_m11; m12=_m12; m13=_m13; m14=_m14;
    m21=_m21; m22=_m22; m23=_m23; m24=_m24;
    m31=_m31; m32=_m32; m33=_m33; m34=_m34;
    m41=_m41; m42=_m42; m43=_m43; m44=_m44;
}

  
//-------------------------------------------------------------------
inline void rpMatrix::copyTo(rpMatrix& m)
{
    memcpy(&m,this,sizeof(rpMatrix));
}    
//-------------------------------------------------------------------
inline float rpMatrix::det(void)
{
    return
	    (m11*m22 - m12*m21)*(m33*m44 - m34*m43)
	   -(m11*m23 - m13*m21)*(m32*m44 - m34*m42)
	   +(m11*m24 - m14*m21)*(m32*m43 - m33*m42)
	   +(m12*m23 - m13*m22)*(m31*m44 - m34*m41)
	   -(m12*m24 - m14*m22)*(m31*m43 - m33*m41)
	   +(m13*m24 - m14*m23)*(m31*m42 - m32*m41);
}
//-------------------------------------------------------------------
inline void rpMatrix::invert(void)
{
	float s = det();	
    if (s == 0.0) return;	
    s = 1/s;
    set(
        s*(m22*(m33*m44 - m34*m43) + m23*(m34*m42 - m32*m44) + m24*(m32*m43 - m33*m42)),
        s*(m32*(m13*m44 - m14*m43) + m33*(m14*m42 - m12*m44) + m34*(m12*m43 - m13*m42)),
        s*(m42*(m13*m24 - m14*m23) + m43*(m14*m22 - m12*m24) + m44*(m12*m23 - m13*m22)),
        s*(m12*(m24*m33 - m23*m34) + m13*(m22*m34 - m24*m32) + m14*(m23*m32 - m22*m33)),
        s*(m23*(m31*m44 - m34*m41) + m24*(m33*m41 - m31*m43) + m21*(m34*m43 - m33*m44)),
        s*(m33*(m11*m44 - m14*m41) + m34*(m13*m41 - m11*m43) + m31*(m14*m43 - m13*m44)),
        s*(m43*(m11*m24 - m14*m21) + m44*(m13*m21 - m11*m23) + m41*(m14*m23 - m13*m24)),
        s*(m13*(m24*m31 - m21*m34) + m14*(m21*m33 - m23*m31) + m11*(m23*m34 - m24*m33)),
        s*(m24*(m31*m42 - m32*m41) + m21*(m32*m44 - m34*m42) + m22*(m34*m41 - m31*m44)),
        s*(m34*(m11*m42 - m12*m41) + m31*(m12*m44 - m14*m42) + m32*(m14*m41 - m11*m44)),
        s*(m44*(m11*m22 - m12*m21) + m41*(m12*m24 - m14*m22) + m42*(m14*m21 - m11*m24)),
        s*(m14*(m22*m31 - m21*m32) + m11*(m24*m32 - m22*m34) + m12*(m21*m34 - m24*m31)),
        s*(m21*(m33*m42 - m32*m43) + m22*(m31*m43 - m33*m41) + m23*(m32*m41 - m31*m42)),
        s*(m31*(m13*m42 - m12*m43) + m32*(m11*m43 - m13*m41) + m33*(m12*m41 - m11*m42)),
        s*(m41*(m13*m22 - m12*m23) + m42*(m11*m23 - m13*m21) + m43*(m12*m21 - m11*m22)),
        s*(m11*(m22*m33 - m23*m32) + m12*(m23*m31 - m21*m33) + m13*(m21*m32 - m22*m31)));
}

//-------------------------------------------------------------------
inline void rpMatrix::ident(void)
{
    memcpy(this,matrix_ident,sizeof(matrix_ident));
}

//-------------------------------------------------------------------
inline void rpMatrix::transpose(rpMatrix &M)
{
	M.m11 = m11;	M.m12 = m21;	M.m13 = m31;	M.m14 = m41;
	M.m21 = m12;	M.m22 = m22;	M.m23 = m32;	M.m24 = m42;
	M.m31 = m13;	M.m32 = m23;	M.m33 = m33;	M.m34 = m43;
	M.m41 = m14;	M.m42 = m24;	M.m43 = m34;	M.m44 = m44;
}


//-------------------------------------------------------------------
inline void rpMatrix::translate(float x, float y, float z)
{
    m41 = x;
    m42 = y;
    m43 = z;
}
//-------------------------------------------------------------------
inline void rpMatrix::move(float x, float y, float z)
{
    m41 += x;
    m42 += y;
    m43 += z;
}
//-------------------------------------------------------------------
inline void rpMatrix::rotateX(float a)
{
    float *m_0 = (float *) &(this->m11);
    float c    = (float) cos(a);
    float s    = (float) sin(a);
    int i;
    for (i=0; i<4; i++) {
        float mi1 = m0(i,1);
        float mi2 = m0(i,2);
        m0(i,1) = mi1*c + mi2*-s;
        m0(i,2) = mi1*s + mi2*c;
    }
}
//-------------------------------------------------------------------
inline void rpMatrix::rotateY(float a)
{
    float *m_0 = (float *) &(this->m11);
    float c    = (float) cos(a);
    float s    = (float) sin(a);
    int i;
    for (i=0; i<4; i++) {
        float mi0 = m0(i,0);
        float mi2 = m0(i,2);
        m0(i,0) = mi0*c + mi2*s;
        m0(i,2) = mi0*-s + mi2*c;
    }
}
//-------------------------------------------------------------------
inline void rpMatrix::rotateZ(float a)
{
    float *m_0 = (float *) &(this->m11);
    float c    = (float) cos(a);
    float s    = (float) sin(a);
    int i;
    for (i=0; i<4; i++) {
        float mi0 = m0(i,0);
        float mi1 = m0(i,1);
        m0(i,0) = mi0*c + mi1*-s;
        m0(i,1) = mi0*s + mi1*c;
    }
}
//-------------------------------------------------------------------
inline void rpMatrix::scale(float x, float y, float z)
{
    float *m_0 = (float *) &(this->m11);
    int i;
    for (i=0; i<4; i++) {
        m0(i,0) *= x;
        m0(i,1) *= y;
        m0(i,2) *= z;
    }
}
//-------------------------------------------------------------------
inline void rpMatrix::setRvec(float x, float y, float z, float a)
{
 
    float c = (float) cos(a);
    float s = (float) sin(a);
    float l = (x*x + y*y + z*z);
    if (l > 0.0) {
        x /= l;
        y /= l;
        z /= l;
    }

	m11 = c + (1.0f-c) * x * x;
	m12 = (1.0f-c) * x * y - s * z;
	m13 = (1.0f-c) * z * x + s * y;
	m14 = 0.0f;
	m21 = (1.0f-c) * x * y + s * z;
	m22 = c + (1.0f-c) * y * y;
	m23 = (1.0f-c) * y * z - s * x;
    m24 = 0.0f;
	m31 = (1.0f-c) * z * x - s * y;
	m32 = (1.0f-c) * y * z + s * x;
	m33 = c + (1.0f-c) * z * z;
    m34 = 0.0f;
    m41 = 0.0f;
    m42 = 0.0f;
    m43 = 0.0f;
    m44 = 1.0f;
}

//-------------------------------------------------------------------
inline void rpMatrix::mult(const rpMatrix& _m1)
{
    float *m_0 = (float *) &(this->m11);
    float *m_1 = (float *) &(_m1.m11);
    int i;
    for (i=0; i<4; i++) {
        float mi0 = m0(i,0);
        float mi1 = m0(i,1);
        float mi2 = m0(i,2);
        float mi3 = m0(i,3);
        m0(i,0) = mi0*m1(0,0) + mi1*m1(1,0) + mi2*m1(2,0) + mi3*m1(3,0);
        m0(i,1) = mi0*m1(0,1) + mi1*m1(1,1) + mi2*m1(2,1) + mi3*m1(3,1);
        m0(i,2) = mi0*m1(0,2) + mi1*m1(1,2) + mi2*m1(2,2) + mi3*m1(3,2);
        m0(i,3) = mi0*m1(0,3) + mi1*m1(1,3) + mi2*m1(2,3) + mi3*m1(3,3);
    }
}    
//-------------------------------------------------------------------
//  (m14==m24==m34==0, m44==1)
//-------------------------------------------------------------------
inline void rpMatrix::multSimple(const rpMatrix& _m1)
{
    float *m_0 = (float *) &(this->m11);
    float *m_1 = (float *) &(_m1.m11);
    int i;
    for (i=0; i<4; i++) {
        float mi0 = m0(i,0);
        float mi1 = m0(i,1);
        float mi2 = m0(i,2);
        m0(i,0) = mi0*m1(0,0) + mi1*m1(1,0) + mi2*m1(2,0);
        m0(i,1) = mi0*m1(0,1) + mi1*m1(1,1) + mi2*m1(2,1);
        m0(i,2) = mi0*m1(0,2) + mi1*m1(1,2) + mi2*m1(2,2);
    }
    m0(3,0) += m1(3,0);
    m0(3,1) += m1(3,1);
    m0(3,2) += m1(3,2);
    m0(0,3) = 0.0f;
    m0(1,3) = 0.0f;
    m0(2,3) = 0.0f;
    m0(3,3) = 1.0f;
}



#undef m0
#undef m1
//-------------------------------------------------------------------
#endif

