/*****************************************************************************\
 *
 * rpLib	- graphic library for rapid development of graphical applications
 *
 * Author:	Pavel Gunia (xgunia01 @ stud.fit.vutbr.cz)
 *
 * ----------------------------------------------------------------------------
 *
 * THIS SOFTWARE IS NOT COPYRIGHTED
 *
 * This source code is offered for use in the public domain.
 * You may use, modify or distribute it freely.
 *
 * This source code is distributed in the hope that it will be useful but
 * WITHOUT ANY WARRANTY.  ALL WARRANTIES, EXPRESS OR IMPLIED ARE HEREBY
 * DISCLAIMED.  This includes but is not limited to warranties of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *
 * If you find the source code useful, authors will kindly welcome
 * if you give them credit and keep their names with their source code,
 * but do not feel to be forced to do so.
 *
\*****************************************************************************/


#ifndef RPPHYSICSNODE_H
#define RPPHYSICSNODE_H
#include "rpbase.h"
#include "rpMatrix.h"
#include "rpVector.h"
#include "rpPhysicsCollision.h"

class rpSceneNode;

class rpPhysicsNode :
	public rpBase
{
public:
	rpPhysicsNode(rpPhysicsWorld *, rpPhysicsCollision *);
	virtual ~rpPhysicsNode(void);

	// link to a scene node
    void	setSceneNode(rpSceneNode *);
	rpSceneNode	*getSceneNode() {return affected_node;}

	// verification
	bool	isFine();

	// 'force' methods
	// virtual methods, override for your own physical rules like gravity or so
	virtual void	applyForceAndTorque();
	virtual void	applyTransform(rpMatrix *);


	// 'newton' methods
	void	setMassMatrix(f32 mass, f32 Ixx, f32 Iyy,f32 Izz);
	void	getMassMatrix(f32& mass, f32& Ixx, f32& Iyy,f32& Izz);

	void	setMatrix(rpMatrix *);
	void	getMatrix(rpMatrix *);
	
	void	setForce(rpVector3D &force);
	void	addForce(rpVector3D &force);
	void	getForce(rpVector3D &);
	
	void	setTorque(rpVector3D &torque);
	void	addTorque(rpVector3D &torque);
	void	getTorque(rpVector3D&);

	void	setVelocity(rpVector3D &velocity);
	void	getVelocity(rpVector3D&);

	void	setOmega(rpVector3D &omega);
	void	getOmega(rpVector3D &);

	void	setLinearDamping(const f32);
	f32		getLinearDamping();
	void	setAngularDamping(rpVector3D &);
	void	getAngularDamping(rpVector3D &);

    
	//
	float weight;


protected:
	rpPhysicsWorld*		pWorld;
	NewtonBody*			nBody;
	
	rpSceneNode*		affected_node;
};


#endif
