/*****************************************************************************\
 *
 * rpLib	- graphic library for rapid development of graphical applications
 *
 * Author:	Pavel Gunia (xgunia01 @ stud.fit.vutbr.cz)
 *
 * ----------------------------------------------------------------------------
 *
 * THIS SOFTWARE IS NOT COPYRIGHTED
 *
 * This source code is offered for use in the public domain.
 * You may use, modify or distribute it freely.
 *
 * This source code is distributed in the hope that it will be useful but
 * WITHOUT ANY WARRANTY.  ALL WARRANTIES, EXPRESS OR IMPLIED ARE HEREBY
 * DISCLAIMED.  This includes but is not limited to warranties of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *
 * If you find the source code useful, authors will kindly welcome
 * if you give them credit and keep their names with their source code,
 * but do not feel to be forced to do so.
 *
\*****************************************************************************/


#ifndef RPSCENEMANAGER_H
#define RPSCENEMANAGER_H


#include "rpbase.h"
#include "rpResourceManager.h"
#include "rpVideoDriver.h"
#include "rpTexture.h"
#include "rpColor.h"
#include "rpPhysicsCollision.h"
#include "rpPhysicsNode.h"

#include <deque>
#include <algorithm>


enum E_RENDERPASS_TYPE {
	E_RPT_PREGEOMETRY = 0,		// all that must be rendered before the actual geometry like lights
	E_RPT_GEOMETRY	};			// geometry rendering pass


class rpSceneNode;
class rpCameraNode;
class rpGeometryNode;
class rpLightNode;

class rpSceneManager :
	public rpBase
{
public:
	rpSceneManager(rpVideoDriver *);
	~rpSceneManager(void);

	rpResourceManager	*resource_manager;
	rpVideoDriver		*video_driver;
	rpSceneNode			*root_node;

	// resources methods
	rpGeometryBuffer*	getGeometryBuffer(const c8*);

	// scene creation methods
	rpGeometryNode*		newGeometryNode(rpSceneNode *_parent, const c8* _file, const c8* _texture_file, bool hbuffer = false);
	rpLightNode*		newLightNode(rpSceneNode *_parent);
	
	rpCameraNode*		newCameraNode(rpSceneNode *_parent);

	rpSceneNode*		newSceneNode(rpSceneNode *_parent);

	// camera
	void				setCamera(rpCameraNode *);
	rpCameraNode*		active_camera;

	// physics
	rpPhysicsWorld*			physics_world;
	rpPhysicsWorld*			newPhysicsWorld();
	rpPhysicsCollision*		newPhysicsCollision(E_PCOLLISION_TYPE, f32,f32,f32,rpMatrix* = NULL);
	rpPhysicsCollision*		newPhysicsCollision(E_PCOLLISION_TYPE, f32,f32,rpMatrix* = NULL);
	rpPhysicsCollision*		newPhysicsCollision(E_PCOLLISION_TYPE, rpGeometryBuffer *,rpMatrix* = NULL);

	rpPhysicsNode*			newPhysicsNode(rpPhysicsCollision*, rpSceneNode *);

	void					updatePhysics();
	void					updatePhysics(f32 step);

	// RENDER METHODS
	bool	beginScene(bool backBuffer, bool zBuffer, rpColorf color);
	bool	endScene();
	void	drawAll();

	// render pass methods
    void	registerNode(const E_RENDERPASS_TYPE, rpSceneNode *, void *key);

	// containers
	struct sDefContainer {
		rpTexture*		key;
		rpSceneNode*	node;
		sDefContainer(){ key = NULL; node = NULL; }
		sDefContainer(rpTexture *K, rpSceneNode *N){ key = K; node = N; }
	};

	std::deque<rpSceneNode *>				pre_geometry_list;
	std::deque<rpSceneNode *>::iterator		pre_geometry_list_it;

	std::deque<sDefContainer>				default_list;
	std::deque<sDefContainer>::iterator		default_list_it;

};

#endif