/*****************************************************************************\
 *
 * rpLib	- graphic library for rapid development of graphical applications
 *
 * Author:	Pavel Gunia (xgunia01 @ stud.fit.vutbr.cz)
 *
 * ----------------------------------------------------------------------------
 *
 * THIS SOFTWARE IS NOT COPYRIGHTED
 *
 * This source code is offered for use in the public domain.
 * You may use, modify or distribute it freely.
 *
 * This source code is distributed in the hope that it will be useful but
 * WITHOUT ANY WARRANTY.  ALL WARRANTIES, EXPRESS OR IMPLIED ARE HEREBY
 * DISCLAIMED.  This includes but is not limited to warranties of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *
 * If you find the source code useful, authors will kindly welcome
 * if you give them credit and keep their names with their source code,
 * but do not feel to be forced to do so.
 *
\*****************************************************************************/


#ifndef RPSCENENODE_H
#define RPSCENENODE_H

#include "rpbase.h"
#include "rpSceneManager.h"
#include "rpMatrix.h"
#include "rpVector.h"
#include <deque>

enum E_AXIS_SELECT {
	E_AXIS_X = 0,
	E_AXIS_Y,
	E_AXIS_Z };


class rpSceneNode :
	public rpBase
{
public:
	rpSceneNode(rpSceneNode *, rpSceneManager *);
	virtual ~rpSceneNode(void);

	//scene manager
	rpSceneManager *scene_manager;

	// transformation
	rpVector3D		position;
	rpVector3D		rotation;

	rpMatrix		transformation;
	rpMatrix		absolute_transformation;
	bool			transformation_changed;
	
	void			rotate(E_AXIS_SELECT, f32);
	void			rotate(f32, f32, f32);
	void			rotateRel(E_AXIS_SELECT, f32);
	void			rotateRel(f32, f32, f32);

	void			move(E_AXIS_SELECT, f32);
	void			move(f32, f32, f32);
	void			moveRel(E_AXIS_SELECT, f32);
	void			moveRel(f32, f32, f32);
	
	
	void			updateTransformation() {
		if (transformation_changed) {
			transformation.ident();
			transformation.rotateX(rotation.x);
			transformation.rotateY(rotation.y);
			transformation.rotateZ(rotation.z);
			transformation.move(position.x, position.y,position.z);
			transformation_changed = false;
		}
	}
	


	
	// visibility
	bool			visible;

	// HIERARCHY STUFF
	rpSceneNode*	parent;
	void			addChildNode(rpSceneNode *);

	void			deleteChildNode(rpSceneNode *);

	// children list
	std::deque<rpSceneNode *>	children;
	std::deque<rpSceneNode *>::iterator	children_it;

	

	// RENDERING METHODS

	virtual void preRender();
	virtual void Render();

};

#endif