/*****************************************************************************\
 *
 * rpLib	- graphic library for rapid development of graphical applications
 *
 * Author:	Pavel Gunia (xgunia01 @ stud.fit.vutbr.cz)
 *
 * ----------------------------------------------------------------------------
 *
 * THIS SOFTWARE IS NOT COPYRIGHTED
 *
 * This source code is offered for use in the public domain.
 * You may use, modify or distribute it freely.
 *
 * This source code is distributed in the hope that it will be useful but
 * WITHOUT ANY WARRANTY.  ALL WARRANTIES, EXPRESS OR IMPLIED ARE HEREBY
 * DISCLAIMED.  This includes but is not limited to warranties of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *
 * If you find the source code useful, authors will kindly welcome
 * if you give them credit and keep their names with their source code,
 * but do not feel to be forced to do so.
 *
\*****************************************************************************/


#ifndef RPTEXTURE_H
#define RPTEXTURE_H

#include "rpTypes.h"
#include "rpBase.h"
#include "rpImage.h"



//! Enumeration flags telling the video driver in which format textures
//! should be created.
enum E_TEXTURE_CREATION_FLAG
{
	E_TCF_ALWAYS_16_BIT = 0x00000001,

	
	E_TCF_ALWAYS_32_BIT = 0x00000002,

	
	E_TCF_OPTIMIZED_FOR_QUALITY  = 0x00000004,

	
	E_TCF_OPTIMIZED_FOR_SPEED = 0x00000008,

	
	E_TCF_CREATE_MIP_MAPS = 0x00000010,

	
	E_TCF_FORCE_32_BIT_DO_NOT_USE = 0x7fffffff,
};

// interface for video driver dependent texture
class rpTexture : public rpBase
{
public:

	
	virtual ~rpTexture() {};

	
	virtual void* lock() = 0;

	
	virtual void unlock() = 0;

	
	virtual const dimension2d<i32>& getOriginalSize() = 0;

	
	virtual const dimension2d<i32>& getSize() = 0;

	virtual E_COLOR_FORMAT getColorFormat() = 0;

	
	virtual i32 getPitch() = 0;

	
	virtual void regenerateMipMapLevels() = 0;

	
	virtual void getFromImage(rpImage *) = 0;

	
	virtual void applyToDriver() = 0;


};






#endif