/*****************************************************************************\
 *
 * rpLib	- graphic library for rapid development of graphical applications
 *
 * Author:	Pavel Gunia (xgunia01 @ stud.fit.vutbr.cz)
 *
 * ----------------------------------------------------------------------------
 *
 * THIS SOFTWARE IS NOT COPYRIGHTED
 *
 * This source code is offered for use in the public domain.
 * You may use, modify or distribute it freely.
 *
 * This source code is distributed in the hope that it will be useful but
 * WITHOUT ANY WARRANTY.  ALL WARRANTIES, EXPRESS OR IMPLIED ARE HEREBY
 * DISCLAIMED.  This includes but is not limited to warranties of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *
 * If you find the source code useful, authors will kindly welcome
 * if you give them credit and keep their names with their source code,
 * but do not feel to be forced to do so.
 *
\*****************************************************************************/


#ifndef RPTEXTUREOPENGL_H
#define RPTEXTUREOPENGL_H

#include "rptexture.h"
#include "include/gl/glut.h"
#include "include/gl/gl.h"
#include "include/gl/glext.h"

class rpTextureOpenGL :
	public rpTexture
{
public:
	
	rpTextureOpenGL(rpImage* surface, bool generateMipLevels);
	
	
	virtual ~rpTextureOpenGL();

	
	virtual void* lock();

	
	virtual void unlock();

	
	virtual const dimension2d<i32>& getOriginalSize();

	
	virtual const dimension2d<i32>& getSize();


	virtual E_COLOR_FORMAT getColorFormat();

	// returns pitch of texture (in bytes)
	virtual i32 getPitch();

	// return open gl texture name
	GLuint getOpenGLTextureName();


	virtual void regenerateMipMapLevels();

	// get from an rpImage object
	virtual void getFromImage(rpImage *);

	// apply to video driver
	virtual void applyToDriver();

private:

	void getImageData(rpImage* image);

	// copies the the texture into an open gl texture.
	void copyTexture();
	
	// returns the size of a texture which would be the optimize size for rendering it
	inline i32 getTextureSizeFromSurfaceSize(i32 size);

	
	inline bool testError();

	dimension2d<i32> ImageSize;
	dimension2d<i32> OriginalSize;
	i32 Pitch;
	bool SufaceHasSameSize; // true if Surface has the same dimension as texture.
	i32* ImageData;
	
	GLuint TextureName;
	bool hasMipMaps;


};


#endif