/*****************************************************************************\
 *
 * rpLib	- graphic library for rapid development of graphical applications
 *
 * Author:	Pavel Gunia (xgunia01 @ stud.fit.vutbr.cz)
 *
 * ----------------------------------------------------------------------------
 *
 * THIS SOFTWARE IS NOT COPYRIGHTED
 *
 * This source code is offered for use in the public domain.
 * You may use, modify or distribute it freely.
 *
 * This source code is distributed in the hope that it will be useful but
 * WITHOUT ANY WARRANTY.  ALL WARRANTIES, EXPRESS OR IMPLIED ARE HEREBY
 * DISCLAIMED.  This includes but is not limited to warranties of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *
 * If you find the source code useful, authors will kindly welcome
 * if you give them credit and keep their names with their source code,
 * but do not feel to be forced to do so.
 *
\*****************************************************************************/


#ifndef RPVIDEODRIVEROPENGL_H
#define RPVIDEODRIVEROPENGL_H


#include "rpvideodriver.h"
#include "rpLightNode.h"

#include "include/gl/glut.h"
#include "include/gl/gl.h"
#include "include/gl/glext.h"

#ifdef WIN32
// include windows headers for HWND
#include <windows.h>
#pragma comment(lib, "OpenGL32.lib")
#pragma comment(lib, "GLu32.lib")
#pragma comment(lib, "GLut32.lib")
#endif

#ifdef LINUX				//TODO: linux constructors
#define GL_GLEXT_LEGACY 1
#include <X11/Xlib.h>
#endif // LINUX


class rpVideoDriverOpenGL :
	public rpVideoDriver
{
public:

	#ifdef WIN32
	rpVideoDriverOpenGL(const dimension2d<i32>& _screenSize, HWND _window, bool _fullscreen,
			bool _stencilBuffer, bool vsync);
	#endif

	virtual ~rpVideoDriverOpenGL(void);



	// METHODS
	
	//! presents the rendered scene on the screen, returns false if failed
	virtual bool		endScene();

	//! clears the zbuffer
	virtual bool		beginScene(bool backBuffer, bool zBuffer, rpColorf color);

	//! 
	virtual bool		queryFeature(E_VIDEO_DRIVER_FEATURE feature);

	//! transformation
	virtual void		setTransform(E_TRANSFORMATION_STATE state, rpMatrix& mat);
	virtual rpMatrix	getTransform(E_TRANSFORMATION_STATE state);

	rpMatrix			transformation[E_TS_COUNT];

	//! get rid of all textures
	virtual void		removeAllTextures(){}
	
	//!
	virtual void		setViewPort(i32, i32, i32, i32){}
	
	//! set device-dependent fog attributes
	virtual void		setFog(rpColor color=rpColor(0,255,255,255), bool linearFog=true, f32 start=50.0f, f32 end=100.0f, 
							f32 density=0.01f, bool pixelFog=false, bool rangeFog=false) {}

	//! Returns the size of the screen or render window.
	//virtual rpVector2D<i32> getScreenSize();

	//!
	virtual void		setTextureCreationFlag(E_TEXTURE_CREATION_FLAG flag, bool enabled){}

	//! 
	virtual void	setTexture(rpTexture *);

	// device dependent texture handling
	virtual rpTexture*	newTexture();

	// device dependent vertex buffers
	virtual rpGeometryBuffer*	newGeometryBuffer();

	// render device dependent geometry buffer
	virtual void renderGeometryBuffer(rpGeometryBuffer *);

	// handle lights
	i32				last_set_light;
	virtual void	addDynamicLight(rpLightNode *);
	virtual void	removeDynamicLights();


	void loadExtensions();

private:
	#ifdef WIN32		
		HDC HDc; // Private GDI Device Context
		HWND Window;
		HGLRC HRc; // Permanent Rendering Context
	#endif

	// optimized texture handling stuff
    rpTexture *last_texture;
	
	// enumeration for rendering modes such as 2d and 3d for minizing the switching of renderStates.
	enum E_RENDER_MODE
	{
		E_RM_NONE = 0,	// no render state has been set yet.
		E_RM_2D,			// 2d drawing rendermode
		E_RM_3D,			// 3d rendering mode
	};
	// current rendering pass
	E_RENDER_MODE CurrentRenderMode;

	// extensions
	bool ResetRenderStates; // bool to make all renderstates be reseted if set.
	bool Transformation3DChanged;
	bool MultiTextureExtension;
	bool ClampTexture;
	bool StencilBuffer;
	bool ARBVertexProgramExtension; //GL_ARB_vertex_program
	bool ARBFragmentProgramExtension; //GL_ARB_fragment_program

	GLint MaxTextureUnits;

	#ifdef _WIN32
		PFNGLACTIVETEXTUREARBPROC pGlActiveTextureARB;
		PFNGLCLIENTACTIVETEXTUREARBPROC	pGlClientActiveTextureARB;

		typedef BOOL (APIENTRY *PFNWGLSWAPINTERVALFARPROC)(int);
		PFNWGLSWAPINTERVALFARPROC wglSwapIntervalEXT;

		PFNGLGENPROGRAMSARBPROC pGlGenProgramsARB;
		PFNGLBINDPROGRAMARBPROC pGlBindProgramARB;
		PFNGLPROGRAMSTRINGARBPROC pGlProgramStringARB;
		PFNGLDELETEPROGRAMSNVPROC pglDeleteProgramsARB;
		PFNGLPROGRAMLOCALPARAMETER4FVARBPROC pglProgramLocalParameter4fvARB;
	#endif

};

#endif