/*****************************************************************************\
 *
 * rpLib	- graphic library for rapid development of graphical applications
 *
 * Author:	Pavel Gunia (xgunia01 @ stud.fit.vutbr.cz)
 *
 * ----------------------------------------------------------------------------
 *
 * THIS SOFTWARE IS NOT COPYRIGHTED
 *
 * This source code is offered for use in the public domain.
 * You may use, modify or distribute it freely.
 *
 * This source code is distributed in the hope that it will be useful but
 * WITHOUT ANY WARRANTY.  ALL WARRANTIES, EXPRESS OR IMPLIED ARE HEREBY
 * DISCLAIMED.  This includes but is not limited to warranties of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *
 * If you find the source code useful, authors will kindly welcome
 * if you give them credit and keep their names with their source code,
 * but do not feel to be forced to do so.
 *
\*****************************************************************************/


#include "include\rpapplicationwin32.h"
#include "include\rpVideoDriverOpenGl.h"

#ifdef WIN32

rpApplicationWin32 *this_application = NULL;


// window procedure for the application
LRESULT CALLBACK WndProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
{
	#ifndef WM_MOUSEWHEEL
	#define WM_MOUSEWHEEL 0x020A
	#endif
	#ifndef WHEEL_DELTA                     
	#define WHEEL_DELTA 120
	#endif

	rpEvent	event;

	switch (message) 
	{
	case WM_PAINT:
		{
			PAINTSTRUCT ps;
			HDC hdc = BeginPaint(hWnd, &ps);
			EndPaint(hWnd, &ps);
		}
		return 0;

	case WM_ERASEBKGND:
		return 0;

	case WM_SETCURSOR: 
		return 0; 
		
	case WM_MOUSEWHEEL: 
		event.EventType = E_ET_MOUSE_INPUT_EVENT; 
		event.MouseInput.Wheel = (float)((short)HIWORD(wParam)) / (float)WHEEL_DELTA;
		event.MouseInput.Event = E_MIE_MOUSE_WHEEL;
		event.MouseInput.X = LOWORD(lParam); 
		event.MouseInput.Y = HIWORD(lParam); 

		if (this_application) 
			this_application->postEvent(event); 
		break;

	case WM_LBUTTONDOWN:
        event.EventType = E_ET_MOUSE_INPUT_EVENT;
		event.MouseInput.Event = E_MIE_LMOUSE_PRESSED_DOWN;
		event.MouseInput.X = LOWORD(lParam);
		event.MouseInput.Y = HIWORD(lParam);
		if (this_application) 
			this_application->postEvent(event); 
		return 0;

	case WM_LBUTTONUP:
		event.EventType = E_ET_MOUSE_INPUT_EVENT;
		event.MouseInput.Event = E_MIE_LMOUSE_LEFT_UP;
		event.MouseInput.X = LOWORD(lParam);
		event.MouseInput.Y = HIWORD(lParam);
		if (this_application) 
			this_application->postEvent(event); 
		return 0;

	case WM_RBUTTONDOWN:
		event.EventType = E_ET_MOUSE_INPUT_EVENT;
		event.MouseInput.Event = E_MIE_RMOUSE_PRESSED_DOWN;
		event.MouseInput.X = LOWORD(lParam);
		event.MouseInput.Y = HIWORD(lParam);
		if (this_application) 
			this_application->postEvent(event); 
		return 0;

	case WM_RBUTTONUP:
		event.EventType = E_ET_MOUSE_INPUT_EVENT;
		event.MouseInput.Event = E_MIE_RMOUSE_LEFT_UP;
		event.MouseInput.X = LOWORD(lParam);
		event.MouseInput.Y = HIWORD(lParam);
		if (this_application) 
			this_application->postEvent(event); 
		return 0;

	case WM_MBUTTONDOWN:
		event.EventType = E_ET_MOUSE_INPUT_EVENT;
		event.MouseInput.Event = E_MIE_MMOUSE_PRESSED_DOWN;
		event.MouseInput.X = LOWORD(lParam);
		event.MouseInput.Y = HIWORD(lParam);
		if (this_application) 
			this_application->postEvent(event); 
		return 0;

	case WM_MBUTTONUP:
		event.EventType = E_ET_MOUSE_INPUT_EVENT;
		event.MouseInput.Event = E_MIE_MMOUSE_LEFT_UP;
		event.MouseInput.X = LOWORD(lParam);
		event.MouseInput.Y = HIWORD(lParam);
		if (this_application) 
			this_application->postEvent(event); 
		return 0;

	case WM_MOUSEMOVE:
		event.EventType = E_ET_MOUSE_INPUT_EVENT;
		event.MouseInput.Event = E_MIE_MOUSE_MOVED;
		event.MouseInput.X = LOWORD(lParam);
		event.MouseInput.Y = HIWORD(lParam);
		if (this_application) 
			this_application->postEvent(event); 
		return 0;

	case WM_KEYDOWN:
		{
			event.EventType = E_ET_KEY_INPUT_EVENT;
			event.KeyInput.Key = (E_KEY_CODE)wParam;
			event.KeyInput.PressedDown = true;
			
			BYTE allKeys[256];
			WORD KeyAsc=0;
			GetKeyboardState(allKeys);
			ToAscii(wParam,lParam,allKeys,&KeyAsc,0);

			event.KeyInput.Shift = ((allKeys[VK_SHIFT] & 0x80)!=0);
			event.KeyInput.Control = ((allKeys[VK_CONTROL] & 0x80)!=0);
			event.KeyInput.Char = KeyAsc; //KeyAsc >= 0 ? KeyAsc : 0;

			if (this_application) 
				this_application->postEvent(event); 

			return 0;
		}
	case WM_KEYUP:
		{
			event.EventType = E_ET_KEY_INPUT_EVENT;
			event.KeyInput.Key = (E_KEY_CODE)wParam;
			event.KeyInput.PressedDown = false;
			

			BYTE allKeys[256];
			WORD KeyAsc=0;
			GetKeyboardState(allKeys);
			ToAscii(wParam,lParam,allKeys,&KeyAsc,0);

			event.KeyInput.Shift = ((allKeys[VK_SHIFT] & 0x80)!=0);
			event.KeyInput.Control = ((allKeys[VK_CONTROL] & 0x80)!=0);
			event.KeyInput.Char = KeyAsc; //KeyAsc >= 0 ? KeyAsc : 0;

			if (this_application) 
				this_application->postEvent(event); 

			return 0;
		}

	case WM_SIZE:
		{
		
		}
		return 0;

	case WM_DESTROY:
		PostQuitMessage(0);
		return 0;

	case WM_SYSCOMMAND:
		// prevent screensaver or monitor powersave mode from starting
		if (wParam == SC_SCREENSAVE ||
			wParam == SC_MONITORPOWER)
			return 0;
		break;
	}

	return DefWindowProc(hWnd, message, wParam, lParam);
}


// application constructor
rpApplicationWin32::rpApplicationWin32(E_VIDEO_DRIVER _video_driver_type,
									   int		window_width,
									   int		window_height,
									   bool		fullscreen
									  ):rpApplication()
{
	this_application = this;

	// now create a window for the application
	HINSTANCE hInstance = GetModuleHandle(0);

	#ifdef _DEBUG
	setDebugName("rpApplicationWin32");	
	#endif

	
	const c8* ClassName = "rpApplicationWin32";

	// register Class

	WNDCLASSEX wcex;
	wcex.cbSize = sizeof(WNDCLASSEX); 
	wcex.style			= CS_HREDRAW | CS_VREDRAW;
	wcex.lpfnWndProc	= (WNDPROC)WndProc;
	wcex.cbClsExtra		= 0;
	wcex.cbWndExtra		= 0;
	wcex.hInstance		= hInstance;
	wcex.hIcon			= NULL;
	wcex.hCursor		= LoadCursor(NULL, IDC_ARROW);
	wcex.hbrBackground	= (HBRUSH)(COLOR_WINDOW+1);
	wcex.lpszMenuName	= 0;
	wcex.lpszClassName	= ClassName;
	wcex.hIconSm		= 0;

	RegisterClassEx(&wcex);

	// calculate client size

	RECT clientSize;
	clientSize.top = 0;
	clientSize.left = 0;
	clientSize.right = window_width;
	clientSize.bottom = window_height;
	
	DWORD style = WS_POPUP;

	if (!fullscreen)
		style = WS_SYSMENU | WS_BORDER | WS_CAPTION | WS_CLIPCHILDREN | WS_CLIPSIBLINGS;

	AdjustWindowRect(&clientSize, style, FALSE);

	i32 realWidth = clientSize.right - clientSize.left;
	i32 realHeight = clientSize.bottom - clientSize.top;

	i32 windowLeft = (GetSystemMetrics(SM_CXSCREEN) - realWidth) / 2;
	i32 windowTop = (GetSystemMetrics(SM_CYSCREEN) - realHeight) / 2;

	if (fullscreen)
	{
		windowLeft = 0;
		windowTop = 0;
	}

	// create window

	HWnd = CreateWindow( ClassName, "", style, windowLeft, windowTop, 
				realWidth, realHeight,	NULL, NULL, hInstance, NULL);

	ShowWindow(HWnd , SW_SHOW);
	UpdateWindow(HWnd);

	// fix ugly ATI driver bugs.
	MoveWindow(HWnd, windowLeft, windowTop, realWidth, realHeight, TRUE);


	// create the scene manager and video driver
	// actually scene manager is always the same for any platform but we'll keep it all togehter

	// create and initialize video driver
	// check for compatibility with the operatin system
	video_driver = new rpVideoDriverOpenGL(dimension2d<i32>(window_width, window_height), HWnd, false, false, false);
	// TODO: check compatibility, but now we have OpenGl only

	// create and initialize scene manager
	scene_manager = new rpSceneManager(video_driver);
	
	
	
	// set this as active window
	SetActiveWindow(HWnd);
	SetForegroundWindow(HWnd);



}

rpApplicationWin32::~rpApplicationWin32()
{
	close();
}

// set window caption
void rpApplicationWin32::setCaption(const c8* caption){
	SetWindowText(HWnd, caption);


}


bool rpApplicationWin32::run()
{
	MSG msg;

	bool quit = false;

	while (PeekMessage(&msg, NULL, 0, 0, PM_REMOVE))
	{
		TranslateMessage(&msg);
		DispatchMessage(&msg);

		if (msg.message == WM_QUIT)
			quit = true;
	}

	return !quit;

	return true;
}

#endif	//WIN32