/*****************************************************************************\
 *
 * rpLib	- graphic library for rapid development of graphical applications
 *
 * Author:	Pavel Gunia (xgunia01 @ stud.fit.vutbr.cz)
 *
 * ----------------------------------------------------------------------------
 *
 * THIS SOFTWARE IS NOT COPYRIGHTED
 *
 * This source code is offered for use in the public domain.
 * You may use, modify or distribute it freely.
 *
 * This source code is distributed in the hope that it will be useful but
 * WITHOUT ANY WARRANTY.  ALL WARRANTIES, EXPRESS OR IMPLIED ARE HEREBY
 * DISCLAIMED.  This includes but is not limited to warranties of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *
 * If you find the source code useful, authors will kindly welcome
 * if you give them credit and keep their names with their source code,
 * but do not feel to be forced to do so.
 *
\*****************************************************************************/


#include "include\rpgeometrynode.h"

rpGeometryNode::rpGeometryNode(rpSceneNode *sN, rpSceneManager *sM):rpSceneNode(sN,sM),texture(NULL)
{

}

rpGeometryNode::~rpGeometryNode(void)
{
	geometry_list.clear();

}

void rpGeometryNode::setTexture(rpTexture *tex)
{
	if (tex) {
		tex->grab();
		texture = tex;
	}
}

rpGeometryBuffer *rpGeometryNode::getCurrentGeometry()
{
	// get current buffer
	if (!geometry_list.empty()) {
		return *(geometry_list.begin());
	} else return NULL;
}

void rpGeometryNode::preRender()
{
	if (visible) {

		// register for rendering
		if (scene_manager)
			scene_manager->registerNode(E_RPT_GEOMETRY, this, texture);

		// pre-render children nodes too
		rpSceneNode::preRender();
	}
}
void rpGeometryNode::Render()
{
	// update transformations and all the general stuff
	rpSceneNode::Render();
	
	// actual rendering
	if (scene_manager)
		if (scene_manager->video_driver) {
			
			// apply current transformation to the video driver
			scene_manager->video_driver->setTransform(E_TS_WORLD, absolute_transformation);

			// get current buffer
			if (!geometry_list.empty()) {
				current_geometry = geometry_list.begin();
				scene_manager->video_driver->renderGeometryBuffer((*current_geometry));
			} else
				scene_manager->video_driver->renderGeometryBuffer(NULL);


	}

}