/*****************************************************************************\
 *
 * rpLib	- graphic library for rapid development of graphical applications
 *
 * Author:	Pavel Gunia (xgunia01 @ stud.fit.vutbr.cz)
 *
 * ----------------------------------------------------------------------------
 *
 * THIS SOFTWARE IS NOT COPYRIGHTED
 *
 * This source code is offered for use in the public domain.
 * You may use, modify or distribute it freely.
 *
 * This source code is distributed in the hope that it will be useful but
 * WITHOUT ANY WARRANTY.  ALL WARRANTIES, EXPRESS OR IMPLIED ARE HEREBY
 * DISCLAIMED.  This includes but is not limited to warranties of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *
 * If you find the source code useful, authors will kindly welcome
 * if you give them credit and keep their names with their source code,
 * but do not feel to be forced to do so.
 *
\*****************************************************************************/


#include "include\rpGlGeometryBuffer.h"
#include "include\rpVideoDriverOpenGl.h"
#include "gl/glut.h"
#include "gl/gl.h"

rpGlGeometryBuffer::rpGlGeometryBuffer(i32 initial_vertices, i32 initial_indices, i32 T, rpVideoDriverOpenGL *D)
	:rpGeometryBuffer(T)
{
	vertices.reallocate(initial_vertices);
	indices.reallocate(initial_indices);

	device = D;
}


rpGlGeometryBuffer::~rpGlGeometryBuffer(void)
{
	vertices.clear();
	normals.clear();
	colors.clear();
	texcoords.clear();
	indices.clear();
}

// allocation
void rpGlGeometryBuffer::reallocate(E_ALLOC_TYPE t, ui32 size)
{
	switch (t) {
		case E_AT_VERTICES:
			vertices.reallocate(size);
			break;
		case E_AT_NORMALS:
			normals.reallocate(size);
			break;
		case E_AT_COLORS:
			colors.reallocate(size);
			break;
		case E_AT_TEXCOORDS:
			texcoords.reallocate(size);
			break;
	}
}
void rpGlGeometryBuffer::addVertex(sStandard3f &V)
{
	vertices.push_back(sVertex3f(V.x, V.y, V.z));
	normals.push_back(sVertex3f(V.nx, V.ny, V.nz));
	colors.push_back(sVertex3f(V.cr, V.cg, V.cb));
	texcoords.push_back(sVertex2f(V.u, V.v));
}	

void rpGlGeometryBuffer::addFace(int first, int second, int last)
{
	indices.push_back((unsigned short)first);
	indices.push_back((unsigned short)second);
	indices.push_back((unsigned short)last);

}
void rpGlGeometryBuffer::addIndex(int i)
{
	indices.push_back((unsigned short)i);
}

// prepare for inserting geometry data
void rpGlGeometryBuffer::begin()
{
	vertices.clear();
	normals.clear();
	colors.clear();
	texcoords.clear();
	indices.clear();

}
// building finished
void rpGlGeometryBuffer::finish()
{
	if (hardware_buffer) {
		
		// generate a display list for this buffer
		buffer_ID = glGenLists(1);
		glNewList(buffer_ID, GL_COMPILE);
		
		// disable hardware buffer, so device renders this buffer usual way
		hardware_buffer = false;

		// render the buffer (into the display list)
		device->renderGeometryBuffer(this);

		// finish the list
		glEndList();

		// turn hardware buffer back on
		hardware_buffer = true;
	}


}

sStandard3f rpGlGeometryBuffer::getVertex(ui32 i)
{
	sStandard3f ret_s;
	if (i < vertices.size()) {
		ret_s.x = vertices[i].x;
		ret_s.y = vertices[i].y;
		ret_s.z = vertices[i].z;
	}
	if (i < normals.size()) {
		ret_s.nx = normals[i].x;
		ret_s.ny = normals[i].y;
		ret_s.nz = normals[i].z;
	}
	if (i < colors.size()) {
		ret_s.cr = colors[i].x;
		ret_s.cg = colors[i].y;
		ret_s.cb = colors[i].z;
	}
	if (i < texcoords.size()) {
		ret_s.u = texcoords[i].u;
		ret_s.v = texcoords[i].v;
	}
	return ret_s;

}
sFace3f rpGlGeometryBuffer::getFace(ui32 i)
{
	sFace3f ret_s;
	if (i < indices.size()) {}
		return ret_s;

}

i32	rpGlGeometryBuffer::getVertexCount()
{
	return vertices.size();
}
i32	rpGlGeometryBuffer::getIndexCount()
{
	return indices.size();
}
