/*****************************************************************************\
 *
 * rpLib	- graphic library for rapid development of graphical applications
 *
 * Author:	Pavel Gunia (xgunia01 @ stud.fit.vutbr.cz)
 *
 * ----------------------------------------------------------------------------
 *
 * THIS SOFTWARE IS NOT COPYRIGHTED
 *
 * This source code is offered for use in the public domain.
 * You may use, modify or distribute it freely.
 *
 * This source code is distributed in the hope that it will be useful but
 * WITHOUT ANY WARRANTY.  ALL WARRANTIES, EXPRESS OR IMPLIED ARE HEREBY
 * DISCLAIMED.  This includes but is not limited to warranties of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *
 * If you find the source code useful, authors will kindly welcome
 * if you give them credit and keep their names with their source code,
 * but do not feel to be forced to do so.
 *
\*****************************************************************************/


#include "include\rpresourcemanager.h"
#include "include\rpLoaderBMP.h"


rpResourceManager::~rpResourceManager(void)
{
}

rpGeometryBuffer *rpResourceManager::newGeometryBuffer(const c8 *_file, bool hbuffer)
{
	if (video_driver) {
		// create object
		rpGeometryBuffer *gB = video_driver->newGeometryBuffer();
		gB->hardware_buffer = hbuffer;
		if (_file) {
			
			// register for lookup
			geobuffer_list[_file] = gB;


			// load
			if (gB->hardware_buffer)
				printf("loading resource[hardware geometry buffer]: %s\n", _file);
			else
				printf("loading resource[geometry buffer]: %s\n", _file);
			gB->getFromFile(_file);
		}
		return gB;
	}
	else return NULL;
}

rpGeometryBuffer *rpResourceManager::getGeometryBuffer(const c8 *_file, bool hbuffer)
{

	
	geobuffer_list_it = geobuffer_list.find(_file);
	
	if (geobuffer_list_it !=geobuffer_list.end())	// found
		if ((*geobuffer_list_it).second)
			return ((*geobuffer_list_it).second);

	// else we create a new one
	return newGeometryBuffer(_file, hbuffer);


}

rpTexture *rpResourceManager::newTexture(const c8 *_file)
{
	rpTexture *tex = video_driver->newTexture();
	if (tex) {
		rpLoaderBMP *_loader = new rpLoaderBMP;
		if (_loader) {
			printf("loading resource[texture]: %s\n", _file);
			tex->getFromImage(_loader->loadImage(_file));
		
			// register for lookup
			textures_list[_file] = tex;
		}
	}
	return tex;
}

rpTexture *rpResourceManager::getTexture(const c8 *_file)
{
	textures_list_it = textures_list.find(_file);
	
	if (textures_list_it !=textures_list.end())	// found
		if ((*textures_list_it).second)
			return ((*textures_list_it).second);

	// else we create a new one
	return newTexture(_file);
}