/*****************************************************************************\
 *
 * rpLib	- graphic library for rapid development of graphical applications
 *
 * Author:	Pavel Gunia (xgunia01 @ stud.fit.vutbr.cz)
 *
 * ----------------------------------------------------------------------------
 *
 * THIS SOFTWARE IS NOT COPYRIGHTED
 *
 * This source code is offered for use in the public domain.
 * You may use, modify or distribute it freely.
 *
 * This source code is distributed in the hope that it will be useful but
 * WITHOUT ANY WARRANTY.  ALL WARRANTIES, EXPRESS OR IMPLIED ARE HEREBY
 * DISCLAIMED.  This includes but is not limited to warranties of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *
 * If you find the source code useful, authors will kindly welcome
 * if you give them credit and keep their names with their source code,
 * but do not feel to be forced to do so.
 *
\*****************************************************************************/


#include "include\rpscenenode.h"

rpSceneNode::rpSceneNode(rpSceneNode *P, rpSceneManager *M): visible(true),parent(P), scene_manager(M),
		transformation_changed(true)
{
	// inform the parent node that this is its child
	if (P)
		P->addChildNode(this);

}

rpSceneNode::~rpSceneNode(void)
{
	// let parent know we were destroyed
	if (parent)
		parent->deleteChildNode(this);
	
	// destroy all children
	for (children_it = children.begin();children_it != children.end(); children_it++) {
		if (*children_it)	{
			// makes deletition easier
			(*children_it)->parent = NULL;

			// delete children
			delete (*children_it);
		}
	}
}

void rpSceneNode::addChildNode(rpSceneNode *sN)
{
	if (sN)
		children.push_back(sN);
}
void rpSceneNode::deleteChildNode(rpSceneNode *sN)
{
	if (sN)
		for (children_it = children.begin(); children_it != children.end(); children_it ++)
			if ((*children_it) == sN) {
				children.erase(children_it);
				break;
			}
}

// TRANSFORMATIONS
void rpSceneNode::rotate(E_AXIS_SELECT aS, f32 angle)
{
	switch (aS) {
		case E_AXIS_X:
			rotation.x = angle;break;
		case E_AXIS_Y:
			rotation.y = angle;break;
		case E_AXIS_Z:
			rotation.z = angle;break;
	}
	transformation_changed = true;
}
void rpSceneNode::rotate(f32 ax, f32 ay, f32 az)
{
	rotation.x = ax; rotation.y = ay; rotation.z = az;
	transformation_changed = true;
}

void rpSceneNode::rotateRel(E_AXIS_SELECT aS, f32 angle)
{
	switch (aS) {
		case E_AXIS_X:
			rotation.x += angle;break;
		case E_AXIS_Y:
			rotation.y += angle;break;
		case E_AXIS_Z:
			rotation.z += angle;break;
	}
	transformation_changed = true;
}
void rpSceneNode::rotateRel(f32 ax, f32 ay, f32 az)
{
	rotation.x += ax; rotation.y += ay; rotation.z += az;
	transformation_changed = true;
}

void rpSceneNode::move(E_AXIS_SELECT aS, f32 dist)
{
	switch (aS) {
		case E_AXIS_X:
			position.x = dist;break;
		case E_AXIS_Y:
			position.y = dist;break;
		case E_AXIS_Z:
			position.z = dist;break;
	}
	transformation_changed = true;
}
void rpSceneNode::move(f32 tx, f32 ty, f32 tz)
{
	position.x = tx; position.y = ty; position.z = tz;
	transformation_changed = true;
}
void rpSceneNode::moveRel(E_AXIS_SELECT aS, f32 dist)
{
	switch (aS) {
		case E_AXIS_X:
			position.x += dist;break;
		case E_AXIS_Y:
			position.y += dist;break;
		case E_AXIS_Z:
			position.z += dist;break;
	}
	transformation_changed = true;
}
void rpSceneNode::moveRel(f32 tx, f32 ty, f32 tz)
{
	position.x += tx; position.y += ty; position.z += tz;
	transformation_changed = true;
}


// RENDER PASS
void rpSceneNode::preRender()
{
	// 'pre-render' all children nodes
	if (visible) {
		
		updateTransformation();

		// hierarchy transformations
		transformation.copyTo(absolute_transformation);
		
		// if we have a parent node apply it's transformation to us
		if (parent)
			absolute_transformation.mult(parent->absolute_transformation);

		for (children_it = children.begin();children_it != children.end(); children_it++) {
			if (*children_it)	(*children_it)->preRender();
		}
	}
	// and that's all, rpSceneNode doesn't render anything
}
void rpSceneNode::Render()
{
	   

}