/*****************************************************************************\
 *
 * rpLib	- graphic library for rapid development of graphical applications
 *
 * Author:	Pavel Gunia (xgunia01 @ stud.fit.vutbr.cz)
 *
 * ----------------------------------------------------------------------------
 *
 * THIS SOFTWARE IS NOT COPYRIGHTED
 *
 * This source code is offered for use in the public domain.
 * You may use, modify or distribute it freely.
 *
 * This source code is distributed in the hope that it will be useful but
 * WITHOUT ANY WARRANTY.  ALL WARRANTIES, EXPRESS OR IMPLIED ARE HEREBY
 * DISCLAIMED.  This includes but is not limited to warranties of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *
 * If you find the source code useful, authors will kindly welcome
 * if you give them credit and keep their names with their source code,
 * but do not feel to be forced to do so.
 *
\*****************************************************************************/


#include "include\rpvideodriver.h"
#include <stdio.h>
#include <sys\timeb.h>

rpVideoDriver::rpVideoDriver(const dimension2d<i32>& _screenSize)
: screenSize(_screenSize), viewport(0,0,0,0), primitivesDrawn(0), textureCreationFlags(0)
{
	#ifdef _DEBUG
	setDebugName("rpVideoDriver");
	#endif

	//setFog();

	//setTextureCreationFlag(E_TCF_OPTIMIZED_FOR_SPEED, true);
	//setTextureCreationFlag(E_TCF_CREATE_MIP_MAPS, true);

	viewport = rect<i32>(position2d<i32>(0,0), screenSize);
	
	// offset to what time the driver was initialized
	timeb tt;
	ftime(&tt);
	startup_secs = tt.time;
	startup_msec = tt.millitm;

	msec_treshold = 999;
	msec_duration = 0;
	last_frames_rendered = 0;
	frames_rendered = 0;
	
}

rpVideoDriver::~rpVideoDriver(void)
{
//	removeAllTextures();
}

//! applications must call this method before performing any rendering. returns false if failed.
bool rpVideoDriver::beginScene(bool backBuffer, bool zBuffer, rpColorf color)
{
	
	primitivesDrawn = 0;
	
	return true;
}



//! applications must call this method after performing any rendering. returns false if failed.
bool rpVideoDriver::endScene()
{
	// another frame finished
	frames_rendered ++;

	timeb tt;
	ftime(&tt);
	
	// reached the treshold
	if (msec_duration >= msec_treshold) {
		msec_duration = 0;
		last_frames_rendered = frames_rendered;
		frames_rendered = 0;
		startup_secs = tt.time;
		startup_msec = tt.millitm;
	}
	else {
		
		msec_duration = ((tt.time - startup_secs) * 1000) - startup_msec + tt.millitm;
	}

	return true;
}

//! queries the features of the driver, returns true if feature is available
bool rpVideoDriver::queryFeature(E_VIDEO_DRIVER_FEATURE feature)
{
	return false;
}

i32 rpVideoDriver::getLastFramesCount()
{
	return last_frames_rendered;

}
f32 rpVideoDriver::getLastFramesPerSec()
{
	return (last_frames_rendered * (1000.0f / float(msec_treshold))) ;

}